/* Configuration file for phyCore-BF537 board...
 */

#define CPU_LE
#define CPU_NAME			"BlackFin BF537"

/*
 * The PLATFORM_XXX variable is established in the makefile and used
 * to distinguish between one of several different BF537 based boards...
 */
#if PLATFORM_PCORE537
#define PLATFORM_NAME	"phyCore-BF537"
#elif PLATFORM_BF537
#define PLATFORM_NAME	"ADDS-BF537-EVAL"
#elif PLATFORM_VTCEZ
#define PLATFORM_NAME	"ADDS-BF537-EVAL (VDB)"
#elif PLATFORM_VTC
#define PLATFORM_NAME	"VTC Camera Board"
#else
#error: Invalid/missing PLATFORM in makefile
#endif

#define APPRAMBASE_OVERRIDE 0x100000
#define BOOTROMBASE_OVERRIDE 0x20000000

/* Used by blackfin header files...
 */
#define __ADSPLPBLACKFIN__ 

#define EXCEPTION_HEADING "EVENT"

/* LOOPS_PER_SECOND:
 * Approximately the size of a loop that will cause a 1-second delay.
 * This value can be adjusted by using the "sleep -c" command.
 */
#define LOOPS_PER_SECOND	140000
#define TIMER_TICKS_PER_MSEC 500000

/* T/RX_DMA_DESCRIPTOR_BASE:
 * If defined, they override the default location of the
 * ethernet drivers's dma descriptor tables.  Usually
 * the addresses point to some area in L1 data memory
 * space...
 *
 * Note: I experimented with this and found that it
 * made zero noticeable difference in the ethernet throughput.
 *
#define TX_DMA_DESCRIPTOR_BASE	0xff900000
#define RX_DMA_DESCRIPTOR_BASE	0xff900100
 */

/* T/X_STAT/DATA_BASE:
 * If defined, they override the default location of the
 * ethernet drivers's buffer space.  Usually the 
 * addresses point to some area in L1 data memory.
 *
 * Note: I experimented with this and found that it
 * made zero noticeable difference in the ethernet throughput.
 *
#define TX_STAT_BASE	0xff900400
#define TX_DATA_BASE	0xff900500
#define RX_STAT_BASE	0xff901000
#define RX_DATA_BASE	0xff901800
 */

/* Flash bank configuration:
 * Basic information needed to configure the flash driver.
 * Fill in port specific values here.
 */
#define SINGLE_FLASH_DEVICE 	1
#define FLASH_BANK0_BASE_ADDR  	0x20000000
#define FLASH_BANK0_SIZE		0x00400000
#define FLASH_PROTECT_RANGE  	"0-6"
#define FLASH_BANK0_WIDTH       2
#define FLASH_LARGEST_SECTOR    0x10000
#define FLASH_TIMEOUT			10000000

/* If there is a need to have the range of protected sectors locked (and
 * the flash device driver supports it), then enable this macro...
#define LOCK_FLASH_PROTECT_RANGE
 */

/* TFS definitions:
 * Values that configure the flash space that is allocated to TFS.
 * Fill in port specific values here.
 */
#define TFSSPARESIZE    		FLASH_LARGEST_SECTOR
#define TFSSTART        		(FLASH_BANK0_BASE_ADDR+0x60000)
#define TFSEND          		0x203effff
#define TFSSPARE        		(TFSEND+1)
#define TFSSECTORCOUNT			((TFSSPARE-TFSSTART)/FLASH_LARGEST_SECTOR)
#define TFS_EBIN_ELF    		1
#define TFS_ALTDEVTBL_BASE		&alt_tfsdevtbl_base
#define FLASHBANKS				1

/* Use a blackfin-specific memcpy routine with TFS's loader.
 * This is needed because to load into blackfin's L1 instruction SRAM,
 * you must use DMA.  In this case, the memcpy function will use DMA
 * whenever the destination falls within the range of BFIN_L1I_START
 * and BFIN_L1I_END.
 */
#define USE_ALTERNATE_TFSLD_MEMCPY
#ifdef USE_ALTERNATE_TFSLD_MEMCPY
#define BFIN_L1I_START	0xffa00000
#define BFIN_L1I_END	0xffa13fff
#endif

/* ALLOCSIZE:
 * Specify the size of the memory block (in monitor space) that
 * is to be allocated to malloc in the monitor.  Note that this
 * size can be dynamically increased using the heap command at
 * runtime.
 */
#define ALLOCSIZE 		(64*1024)

/* MONSTACKSIZE:
 * The amount of space allocated to the monitor's stack.
 */
#define MONSTACKSIZE	(8*1024)

#define NO_MONLIB_I2C_STUBS

/* INCLUDE_XXX Macros:
 * Set/clear the appropriate macros depending on what you want
 * to configure in your system.  The sanity of this list is tested
 * through the inclusion of "inc_check.h" at the bottom of this list...
 * When starting a port, set all but INCLUDE_MALLOC, INCLUDE_SHELLVARS,
 * INCLUDE_MEMCMDS and INCLUDE_XMODEM to zero.  Then in small steps
 * enable the following major features in the following order:
 *
 *	INCLUDE_FLASH to test the flash device drivers.
 *	INCLUDE_TFS* to overlay TFS onto the flash. 
 *  INCLUDE_ETHERNET, INCLUDE_TFTP to turn the ethernet interface.
 *
 * All other INCLUDE_XXX macros can be enabled as needed and for the
 * most part, they're hardware independent.
 * Note that for building a very small footprint, INCLUDE_MALLOC and
 * INCLUDE_SHELLVARS can be disabled.
 */
								
#define INCLUDE_MALLOC			1
#define INCLUDE_MEMCMDS         1
#define INCLUDE_SHELLVARS		1
#define INCLUDE_XMODEM          1
#define INCLUDE_DISASSEMBLER    0
#define INCLUDE_UNZIP           1
#define INCLUDE_ETHERNET        1
#define INCLUDE_ICMP			1
#define INCLUDE_IGMP			0
#define INCLUDE_TFTP            1
#define INCLUDE_TFS             1
#define INCLUDE_EDIT            INCLUDE_TFS
#define INCLUDE_FLASH           1
#define INCLUDE_LINEEDIT        1
#define INCLUDE_DHCPBOOT        1
#define INCLUDE_TFSAPI          INCLUDE_TFS
#define INCLUDE_TFSSYMTBL       INCLUDE_TFS
#define INCLUDE_TFSSCRIPT       INCLUDE_TFS
#define INCLUDE_TFSCLI          INCLUDE_TFS
#define INCLUDE_EE              1
#define INCLUDE_GDB             0
#define INCLUDE_STRACE          INCLUDE_TFS
#define INCLUDE_CAST            1
#define INCLUDE_STRUCT          INCLUDE_TFS
#define INCLUDE_REDIRECT        INCLUDE_TFS
#define INCLUDE_QUICKMEMCPY     1
#define INCLUDE_PROFILER        INCLUDE_TFS
#define INCLUDE_BBC             0
#define INCLUDE_MEMTRACE        1
#define INCLUDE_VERBOSEHELP     1
#define INCLUDE_HWTMR	 	    1
#define INCLUDE_PORTCMD	 	    0
#define INCLUDE_USRLVL	 	    INCLUDE_TFS
#define INCLUDE_BFINWATCH	    1
#define INCLUDE_TIMEOFDAY	    1

#if PLATFORM_PCORE537
#define INCLUDE_STOREMAC        0
#else
#define INCLUDE_STOREMAC        1
#endif

#include "inc_check.h"
