/* CLKIN is taken from the CLKIN bit on the CPU.  The BF537 STAMP board
 * has a 25Mhz clock on this line...
 */
#define CLKIN 25000000		/* This is the frequency into the CPU */

/* Set PLL up so that we have a 500Mhz core clock and 100Mhz
 * system clock (refer to pg 20.4 of BF537 HW reference for more
 * info).
 */
#define CORECLKFREQ 500000000	/* 500 Mhz Core clock */
#define SYSCLKFREQ  100000000	/* 100 Mhz System clock */


#define PLL_DF_VAL	0		
#define CSEL_VAL	0		
#define MSEL_VAL    (CORECLKFREQ/CLKIN)	
#define SSEL_VAL	(CORECLKFREQ/SYSCLKFREQ)

/* DRAM stuff...
 */
#define TRAS	5
#define TRP		2

/* Uart stuff...
 */
#define DEFAULT_BAUD_RATE 57600
#define THR		0x00
#define RBR		0x00
#define DLL		0x00
#define DLH		0x04
#define IER		0x04
#define LCR		0x0c
#define MCR		0x10
#define LSR		0x14
#define SCR		0x1c
#define GCTL	0x24

#define UART0_BASE	0xffc00400
#define UART1_BASE	0xffc02000
#define CONSOLE(reg) (UART0_BASE+reg)

/* I-Squared-C (TWI in Blackfin terms) stuff...
 */
#define I2C_SCL_FREQ	400000		/* This is usually either 100K or 400K */
#define I2C_TIMEOUT		1000000	

/* Six LEDs on port 'F'...
 */
#define LED6 0x0800
#define LED5 0x0400
#define LED4 0x0200
#define LED3 0x0100
#define LED2 0x0080
#define LED1 0x0040
#define LED_BITMASK (LED4|LED3)

#define RESETFUNC()	((void(*)())0x20000000)

extern int IFlush(char *addr);
extern int Flush(char *addr);
extern int FlushInv(char *addr);
