#define __ADSPLPBLACKFIN__

#include "cpu.h"
#include "cdefBF537.h"

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned long ulong;

struct xinfo {
	uchar	sno;			/* Sequence number. */
	uchar	pad;			/* Unused, padding. */
	int		xfertot;		/* Running total of transfer. */
	int		pktcnt;			/* Running tally of number of packets processed. */
	long	size;			/* Size of upload. */
	ulong	base;			/* Starting address for data transfer. */
	ulong	dataddr;		/* Running address for data transfer. */
};

/* X/Ymodem protocol: */
#define SOH			0x01
#define STX			0x02
#define EOT			0x04
#define ACK			0x06
#define XNAK		0x15
#define CAN			0x18
#define EOF			0x1a
#define ESC			0x1b
#define NAK_DELAY	1500000

#define PKTLEN		128
#define ENTRY		BASE

void Go(void);
int Xdown(void);
int getPacket(uchar *,struct xinfo *);

long MonStack[32];

void
ledtoggle(void)
{
	volatile int i;

	*pPORTFIO_SET |= LED_BITMASK;
	for(i=0;i<9000000;i++);
	*pPORTFIO_CLEAR |= LED_BITMASK;
	for(i=0;i<5000000;i++);
}

/* devInit():
 * Initialize console...
 */
void
devInit(int baud)
{
	unsigned short tmp;
	unsigned short	lctl, hi, lo;

	*pUART0_GCTL = 1;		/* Set the UCEN bit (enable UART clocks) */
	*pUART0_MCR = 0;
	*pUART0_LCR = 0x03;
	tmp = *pUART0_LSR;
	*pUART0_IER = 0;
	*pUART0_SCR = 0;

	lo = SYSCLKFREQ/(baud*16);
	hi = (lo & 0xff00) >> 8;
	lo &= 0x00ff;

	lctl = *pUART0_LCR;
	*pUART0_LCR = DLAB;
	*pUART0_DLL = lo;
	*pUART0_DLH = hi;
	*pUART0_LCR = lctl;
}

/* target_putchar():
 * When buffer has space available, load the incoming character
 * into the UART.
 */
int
target_putchar(char c)
{
    /* Wait for transmit ready bit */
	while(!(*pUART0_LSR & THRE));

	*pUART0_THR = (unsigned short)c;

    return((int)c);
}

int
target_gotachar(void)
{
	if (*pUART0_LSR & DR)
		return(1);
    return(0);
}
 
int 
target_getchar(void)
{
    return((int)*pUART0_RBR);
}

int
getchar(void)
{
	while((*pUART0_LSR & DR) == 0);
    return((int)*pUART0_RBR);
}

void
memtest(void)
{
	volatile long i, *lp;

	lp = (volatile long *)BASE;

	for(i=0;i<(1024*256);i++) {
		*lp = i;
	}
	for(i=0;i<(1024*256);i++) {
		if (*lp != i) {
			target_putchar('F');
			return;
		}
	}
	target_putchar('P');
}

void
mode7start(void)
{
	int xdone;
	volatile char *cp;
	volatile int c;
	volatile unsigned short stmp;
	volatile unsigned long ltmp;

	/* PLL setup:
	 */
	asm("ssync;");
	*pPLL_CTL = (MSEL_VAL << 9) | PLL_DF_VAL;
	*pSIC_IWR = 1;
	asm("ssync;");
	asm("cli r0;");
	asm("idle;");
	asm("sti r0;");
	asm("ssync;");

	*pPLL_LOCKCNT =	0x0300;
	*pPLL_DIV = (SSEL_VAL) | (CSEL_VAL << 4);
	asm("ssync;");

	*pDTEST_COMMAND = 0;

	/* Memory bank init:
	 */
	*pEBIU_AMBCTL0 = 0x7bb07bb0;
	*pEBIU_AMBCTL1 = 0xffc27bb0;
	asm("ssync;");

	/* SDRAM ctrlr init:
	 * (should be done 100us after reset)
	 * Refer to pg 6-60 of HW reference.
	 */
	for(c=0;c<10000000;c++);
	*pEBIU_SDRRC = (((SYSCLKFREQ/1000) * 64) / 8192) - (TRAS + TRP);
	asm("ssync;");
	*pEBIU_SDBCTL = (EBCAW_10 | EBSZ_64 | EBE);
	asm("ssync;");
	if (*pEBIU_SDSTAT & SDRS)
		*pEBIU_SDGCTL = (PSS | TWR_2 | TRCD_2 | TRP_2 | TRAS_5 | CL_3 | SCTLE);
	else
		*pEBIU_SDGCTL = (TWR_2 | TRCD_2 | TRP_2 | TRAS_5 | CL_3 | SCTLE);
	asm("ssync;");

	*pEBIU_AMGCTL = 0x00ff;
	asm("ssync;");
	
	for(c=0;c<10000;c++);
		*(char *)c = (char)c;

	*pPORT_MUX = 0x0000;

	/* Port F configuration:
	 */
	stmp = *pPORTF_FER;
	*pPORTF_FER = 0x000f;
	*pPORTF_FER = 0x000f;
	*pPORTFIO_DIR = LED_BITMASK;
	*pPORTF_FER = 0x000f;
	*pPORTF_FER = 0x000f;

	/* Port G configuration:
	 */
	stmp = *pPORTG_FER;
	*pPORTG_FER = 0x0000;
	*pPORTG_FER = 0x0000;

	/* Port H configuration:
	 */
	stmp = *pPORTH_FER;
	*pPORTH_FER = 0xffff;
	*pPORTH_FER = 0xffff;

	devInit(DEFAULT_BAUD_RATE);
	xdone = 0;
	while(1) {
		if (target_gotachar()) {
			c = target_getchar();
			if (c == 'x') {
				if (Xdown() > 0)
					xdone = 1;
			}
			else if (c == 't') {
				memtest();
			}
			else if (c == 'g') {
				if (xdone == 1)
					Go();
				else
					target_putchar('?');
			}
			else
				target_putchar(c);
		}
		target_putchar(':');
		ledtoggle();
	}
}

/* getPacket():
 * Used by Xdown to retrieve packets.
 */
int
getPacket(uchar *tmppkt, struct xinfo *xip)
{
	char	*pkt;
	int		i,rcvd;
	uchar	csum, xcsum, seq[2];

	seq[0] = getchar();
	seq[1] = getchar();

	pkt = (char *)xip->dataddr;
	for(i=0;i<PKTLEN;i++)
		*pkt++ = getchar();
	pkt = (char *)xip->dataddr;

	xcsum = (uchar)getchar();
	csum = 0;
	for(i=0;i<PKTLEN;i++)
		csum += *pkt++;
	if (csum != xcsum) {
		target_putchar(XNAK);
		return(0);
	}

	/* Test the sequence number compliment...
	 */
	if ((uchar)seq[0] != (uchar)~seq[1]) {
		target_putchar(XNAK);
		return(0);
	}

	/* Verify that the incoming sequence number is the expected value...
	 */
	if ((uchar)seq[0] !=  xip->sno) {
		/* If the incoming sequence number is one less than the expected
		 * sequence number, then we assume that the sender did not recieve
		 * our previous ACK, and they are resending the previously received
		 * packet.  In that case, we send  ACK and don't process the
		 * incoming packet...
	 	 */
		if ((uchar)seq[0] == xip->sno-1) {
			target_putchar(ACK);
			return(0);
		}

		/* Otherwise, something's messed up...
		 */
		target_putchar(CAN);
		return(-1);
	}

	xip->dataddr += PKTLEN;
	xip->sno++;
	xip->pktcnt++;
	xip->xfertot += PKTLEN;
	target_putchar(ACK);
	return(0);
}

/* Xdown():
 * Called when a transfer from host to target is being made
 * (considered an download).
 */
int
Xdown(void)
{
	int		done = 0;
	struct	xinfo xi;
	char	c, *tmppkt;
	static char pkt[PKTLEN];

	tmppkt = pkt;
	xi.size = 0;
	xi.dataddr = xi.base = BASE;
	xi.sno = 0x01;
	xi.pktcnt = 0;
	xi.xfertot = 0;

	/* Startup synchronization... */
	/* Continuously send XNAK or 'C' until sender responds. */
	while(1) {
		volatile int i;
		target_putchar(XNAK);

		for(i=0;i<NAK_DELAY;i++) {
			if (target_gotachar())
				break;
		}
		if (target_gotachar())
			break;
	}

	while(done == 0) {
		c = (char)getchar();
		if (c == SOH) {		/* 128-byte incoming packet */
			done = getPacket((uchar *)tmppkt,&xi);
		}
		else if (c == EOT) {
			target_putchar(ACK);
			done = xi.xfertot;
		}
		else if (c == EOF) {	/* 0x1a sent by MiniCom, just ignore it. */
			done = 0;
		}
		else {
			done = -1;
		}
	}
	return(done);
}

void
Go(void)
{
	void (*entry)(void);

	entry = (void(*)(void))ENTRY;
	entry();
}
