/*
 *  This source code has been made available to you by IBM on an AS-IS
 *  basis.  Anyone receiving this source is licensed under IBM
 *  copyrights to use it in any way he or she deems fit, including
 *  copying it, modifying it, compiling it, and redistributing it either
 *  with or without modifications.  No license under IBM patents or
 *  patent applications is to be implied by the copyright license.
 *
 *  Any user of this software should understand that IBM cannot provide
 *  technical support for this software and will not be responsible for
 *  any consequences resulting from the use of this software.
 *
 *  Any person who transfers this source code or any derivative work
 *  must include the IBM copyright notice, this paragraph, and the
 *  preceding two paragraphs in the transferred software.
 *
 *  COPYRIGHT   I B M   CORPORATION 1995
 *  LICENSED MATERIAL  -  PROGRAM PROPERTY OF I B M
 *
 *  File Name:   entry.s
 *
 *  Function:   Entry code for the 405 ROM Monitor (openbios)running 
 *              on the Walnut board.
 *
 *  Author:      James Burke  
 *
 *  Change Activity-
 *
 *  Date        Description of Change                                       BY
 *  ---------   ---------------------                                       ---
 *  29-Apr-99   Created                                                     JWB
 *  17-Jun-99   Updated to run with caches on                               JWB
 *  23-Jun-99   Set MCOPT1 BRD_PRF bits to '01' for 16 byte PLB burst read
 *              prefetch to improve SDRAM performance.             JWB
 *  09-Jul-99   Added call to ppcDflush() after rom2ram()                   JWB
 *  14-Aug-99   Moved disabling of PCI addr pipelining for pass1 405GP bug. JWB
 *  24-Aug-99   Added pci ethernet functions to user callable functions.    JWB
 *  07-Sep-99   Moved ppcDflush() call in boot_jump(); leave SDRAM address
 *              pipelining enabled (default) in CNTRL1 reg.                 JWB
 *  22-Sep-99   Updated for sdram_init for 32MB SDRAM DIMM now being 
 *              shipped w/ the board                                        JWB
 *  18-Apr-00   Integrated into the MicroMonitor platform.                  ELS
 *  21-Apr-00   Added INCLUDE_ECC to optionally disable ECC and             ELS
 *              INCLUDE_HERALD to optionally include the startup message.
 *  23-Mar-02	Modified and cleaned up for use on Cogent CSB272		    MJK
 *				Michael J. Kelly - Cogent Computer Systems, Inc
 *              mike@cogcomp.com, www.cogcomp.com
 *
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:  Ed Sutter
 * email:   esutter@lucent.com
 * phone:   908-582-2351
 *
 * This reset code is a combination of code written by several developers.
 * The source code for the IBM "Walnut" BSP was used as a reference for this
 * bootup.
 */

#include "config.h"
#include "arch_ppc.h"
#include "cpu_403.h"
#include "warmstart.h"
#include "cpu.h"

    .extern	MonStack
    .globl  coldstart, warmstart, reset

#include "etheraddr.S"

	.balign 0x100

/* reset:
 * This is the code that is pointed to by the resetvector at 0xfffffffc.
 * If we get here, then we know that we took some kind of "hard" reset.
 */
reset:
coldstart:
    xor     r0,r0,r0            /* Clear R0 */
    addi    r3,r0,INITIALIZE

/* warmstart:
 * This function can be entered either by having previously hit coldstart,
 * or by the monitor source code calling warmstart().  It is assumed to
 * be a function to the C-code, and it is passed a reason for the warmstart.
 * The register which the compiler uses for storage of the calling argument
 * must be saved so that it can be passed to start().
 */
warmstart:
    mr      r15,r3              /* Copy r3 to r15 because between here and */
                                /* the call to start(), r3 is used a lot. */
    xor     r0,r0,r0            /* Clear R0 */
    addi    r4,r0,0x0000        /* Clear and set up some registers.  */
    mtsgr   r4
    mtdcwr  r4
    mtesr   r4                  /* clear Exception Syndrome Reg */
    mttcr   r4                  /* clear Timer Control Reg */
    mtxer   r4                  /* clear Fixed-Point Exception Reg */
    addi    r4,r0,0x1000        /* set ME bit (Machine Exceptions) */
    oris    r4,r4,0x0002        /* set CE bit (Critical Exceptions) */
    mtmsr   r4                  /* change MSR */
    ori     r4,r0,0xffff        /* set r4 to 0xFFFFFFFF (status in the */
                                /* dbsr is cleared by setting bits to 1) */
    mtdbsr  r4                  /* clear/reset the dbsr */

    /* Invalidate i-cache and then enable for defined memory regions. 
     * Do the same for the d-cache. Be aware that the caches may be disabled
     * later in miscLib/main.c based on the user selected cache menu options.
     */
    bl      invalidate_icache
    addis   r4,r0, I_CACHEABLE_REGIONS@h
    ori     r4,r4, I_CACHEABLE_REGIONS@l
    mticcr  r4    
    addis   r4,r0, D_CACHEABLE_REGIONS@h
    ori     r4,r4, D_CACHEABLE_REGIONS@l
    bl      invalidate_dcache
    mtdccr  r4    

    /* Initialize the Control 0 register
	 * Enable CS4-7 as GPIO
     * Set UART1 for CTS/RTS and set the UART0 and UART1 external
     * clock enable to use the external serial clock instead of an  
     * internally derived clock.
     */
    addis   r3,r0,0x00F0        /* Set CS4-7 as GPIO13-16 */
    ori     r3,r3,0x00C0        /* set CTS/RTS for UART1 and set ext */
    							/* clock for UART0 and UART1 */

    mfdcr   r4,cntrl0           /* read CNTRL0 */
    or      r3,r3,r4            /* read-modify-write */
    mtdcr   cntrl0,r3           /* set CNTRL0 */

	// turn led 0 on
    addis   r5,r0,0xef60        /* Base of GPIO Registers  */
    ori     r5,r5,0x0700
		
	addis	r3,r0,0x0000		/* turn all bits low, use tristate to control */
	ori		r3,r3,0x0000
	stw		r3,0x00(r5)			/* clear output bits */
	stw		r3,0x18(r5)			/* clear open drain bits */

	addis	r3,r0,0x0006		/* turn on LED0 and 1 tri-state driver */
	ori		r3,r3,0x0000
	stw		r3,0x04(r5)

    /* Initialize the Control 1 register
	 * Set source of Timer to Internal CPU Clock
	 * Enable the PCI_INT/LWR signal to be LWR
     */
    addis   r5,r0,0x0000        /* Set Timer Source to Internal CPU Clock  */
    ori     r5,r0,0x4000        /* set PCI_INTA/*LWE to *LWE */

    mfdcr   r4,cntrl1         	/* read CNTRL1 */
    or      r5,r5,r4          	/* read-modify-write */
    mtdcr   cntrl1,r5         	/* set CNTRL1 */

    /* Initialize the External Bus Controller for external peripherals */
    bl      ext_busctl_init

#ifdef INIT_SDRAM
    /* Initialize SDRAM Controller if running from boot flash.
     */
    bl      sdram_init
#endif

    /* Clear reservation bit. */
    addis   r4,r0,0x0000
    lwarx   r3,r0,r4            /* get some data/set resv bit */
    stwcx.  r3,r0,r4            /* store out and clear resv bit */

    /* Flush data cache. */
    bl      ppcDflush

SPInit:
    /* initialize the stack pointer (Diab/Data compiler needs 2 words) */
    lis     sp, (MonStack+(MONSTACKSIZE-4))@h
    ori		sp, sp, (MonStack+(MONSTACKSIZE-4))@l
    addi    r7, r0, -8
    and     sp, sp, r7      /* 8-byte aligned (EABI spec) */

#if 0
    /*
    * initialize r2 and r13 according to EABI standard
    * Warning: The monitor cannot use the small data or small const areas.
    *          When an application runs, it takes over R2 and R13 so the
    *          monitor can't use them and allow itself to be called by the
    *          application.  This also means the monitor cannot call the
    *          Diab/Data standard library since those functions do not
    *          avoid these areas.
    */
    lis     r2, (_SDA2_BASE_)@h
    ori		r2, r2, (_SDA2_BASE_)@l
    lis		r13, (_SDA_BASE_)@h
    ori		r13, r13, (_SDA_BASE_)@l
#endif

GoToC:
    bl      invalidate_icache   /* Cache had been enabled previously, so */
    addi    r4,0, 0x0000        /* we need to shut if off prior to entering */
    mticcr  r4                  /* C-level because the C-startup assumes */
    addi    r4,0, 0x0000        /* it starts with cache off, then it will */
    bl      invalidate_dcache   /* appropriately enbable cache when it is */
    mtdccr  r4                  /* ready (after calling FlashInit()). */

    mr      r3,r15              /* Copy r15 to r3. */
	ba		start
	nop

/****************************************************************************
 *
 * Function:     sdram_init
 * Description:  Configures SDRAM memory banks.
 *      Assumes that two 8Mx16 or 16Mx16 SDRAMs are installed.
 */
    .text
    .align 2
    .globl  sdram_init

sdram_init:
    mflr    r31

    /*
     * Set MB0CF for bank 0. (0-64MB) Address Mode 2 = 12 row x 9/10
	 * column x 4 bank
     */
    addi    r4,0,mem_mb0cf
    mtdcr   memcfga,r4
    addis   r4,0,0x0008		/* Base Address = 0, Bank Size = 64MB */
    ori     r4,r4,0x2001	/* Mode 2, Bank Enable On */
    mtdcr   memcfgd,r4

    /*
     * Delay to ensure 200usec have elapsed since reset. Assume worst
     * case that the core is running 200Mhz:
     *   200,000,000 (cycles/sec) X .000200 (sec) = 0x9C40 cycles
     */
    addis   r3,0,0x0000
    ori     r3,r3,0xA000        /* ensure 200usec have passed since reset */
    mtctr   r3
..spinlp2:
    bdnz    ..spinlp2           /* spin loop */

    /*
     * Set memory controller options reg, MCOPT1.
     * Set DC_EN to '1' and BRD_PRF to '01' for 16 byte PLB Burst 
     * read/prefetch.
     */
    addi    r4,0,mem_mcopt1
    mtdcr   memcfga,r4
    addis   r4,0,0x8080         /* set DC_EN=1 */
    ori     r4,r4,0000
    mtdcr   memcfgd,r4

    /*
     * Delay to ensure 10msec have elapsed since reset. This is
     * required for the MPC952 to stabalize. Assume worst
     * case that the core is running 200Mhz:
     *   200,000,000 (cycles/sec) X .010 (sec) = 0x1E8480 cycles
     * This delay should occur before accessing SDRAM.
     */
    addis   r3,0,0x001E
    ori     r3,r3,0x8480        /* ensure 10msec have passed since reset */
    mtctr   r3
..spinlp3:
    bdnz    ..spinlp3           /* spin loop */


    mtlr    r31                     /* restore lr */
    blr

/****************************************************************************
 *
 * Function: invalidate_icache
 */
    .text
    .align 2
    .globl  invalidate_icache

invalidate_icache:
    iccci   r0,r0           /* for 405, iccci invalidates the */ 
    blr                     /* entire I cache  */

/****************************************************************************
 *
 * Function: invalidate_dcache
 */
    .text
    .align 2
    .globl  invalidate_dcache    

invalidate_dcache:    
    addi    r5,0,0x0000     /* clear GPR 6 */
    addi    r7,r0, 256      /* do loop for # of dcache lines */
                            /* NOTE: dccci invalidates both  */
    mtctr   r7              /* ways in the D cache.  Loop 256 */
							/* lines to handle 450GPr as well */
..dcloop:
    dccci   0,r5            /* invalidate line */
    addi    r5,r5, 32       /* bump to next line */
    bdnz    ..dcloop
    blr

/****************************************************************************
 *
 * Function:     ext_busctl_init
 * Description:  Initializes the External Bus Controller as follows:
 * Bank 0 - Flash 
 * Bank 1 - HT6542B PS2 Controller
 * Bank 2 - SED1356 Video Controller
 * Bank 3 - Unused
 * Bank 4 - GPIO - LED0, Low True
 * Bank 5 - GPIO - LED1, Low True
 * Bank 6 - GPIO - PCI IDSEL Slot 0, High True
 * Bank 7 - GPIO - PCI IDSEL Slot 1, High True
 */
    .text
    .align 2
    .globl  ext_busctl_init

ext_busctl_init:

    addis   r4,r0, I_CACHEABLE_REGIONS@h
    ori     r4,r4, I_CACHEABLE_REGIONS@l

    mficcr  r9                      /* get iccr value */
    cmp     cr0,0,r9,r4             /* check if caching already enabled */
    beq     ..icache_on             /* if not, */
    mticcr  r4                      /* enable caching  */
..icache_on:
    addis   r3,0,ext_busctl_init@h  /* store the address of the  */
    ori     r3,r3,ext_busctl_init@l /* ext_busctl_init functn in r3 */
    addi    r4,0,11                 /* set ctr to 10; used to prefetch */
    mtctr   r4                      /* 10 cache lines to fit this function */
                                    /* in cache (gives us 8x10=80 instrctns) */
..ebcloop:
    icbt    r0,r3                   /* prefetch cache line for addr in r3 */
    addi    r3,r3,32                /* move to next cache line */
    bdnz    ..ebcloop               /* continue for 10 cache lines */

/* Delay to ensure all accesses to ROM are complete before changing
 * bank 0 timings. 200usec should be enough.
 *   200,000,000 (cycles/sec) X .000200 (sec) = 0x9C40 cycles
 */
    addis   r3,0,0x0
    ori     r3,r3,0xA000            /* ensure 200usec have passed since reset */
    mtctr   r3
..spinlp:
    bdnz    ..spinlp                /* spin loop */

/*
 * External Bus Configuration Register
 */
	addi 	r4, 0,ebc0cfg
    mtdcr   ebccfga,r4
    addis   r4,0,0xd84c				/* 2048 timeout, no external master */
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

/*
 * Memory Bank 7-3, Unused
 */
    addi    r4,0,pb3cr
    mtdcr   ebccfga,r4
    addis   r4,0,0x0000
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

    addi    r4,0,pb4cr
    mtdcr   ebccfga,r4
    addis   r4,0,0x0000
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

    addi    r4,0,pb5cr
    mtdcr   ebccfga,r4
    addis   r4,0,0x0000
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

    addi    r4,0,pb6cr
    mtdcr   ebccfga,r4
    addis   r4,0,0x0000
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

    addi    r4,0,pb7cr
    mtdcr   ebccfga,r4
    addis   r4,0,0x0000
    ori     r4,r4,0x0000
    mtdcr   ebccfgd,r4

/*
 * Memory Bank 2, SED1356
 */
    addi    r4,0,pb2ap
    mtdcr   ebccfga,r4
    addis   r4,0,0x0586		/* no burst, 11 waits (165ns) + external wait */
    ori     r4,r4,0x0300	/* IMPORTANT: no delay for start of CS, WE or OE */
    mtdcr   ebccfgd,r4

    addi    r4,0,pb2cr
    mtdcr   ebccfga,r4
    addis   r4,0,0xf045
    ori     r4,r4,0xA000
    mtdcr   ebccfgd,r4

/*
 * Memory Bank 1, HT6542B PS2 Controller 
 */
    addi    r4,0,pb1ap
    mtdcr   ebccfga,r4
    addis   r4,0,0x1f86				/* no burst, 64 waits */         
    ori     r4,r4,0x9200
    mtdcr   ebccfgd,r4

    addi    r4,0,pb1cr
    mtdcr   ebccfga,r4
    addis   r4,0,0xf081
    ori     r4,r4,0x8000
    mtdcr   ebccfgd,r4

/*
 * Memory Bank 0, 28F128 x 2 StrataFLASH (8M x 32 = 32Mbytes)
 */
    addi    r4,0,pb0ap
    mtdcr   ebccfga,r4
    addis   r4,0,0x0306			/* no burst, 6 wait, 1 wait cs, 2 wait we/oe */
    ori     r4,r4,0x9200		/* we term 1 clk early, 1 clock hold */
    mtdcr   ebccfgd,r4

    addi    r4,0,pb0cr
    mtdcr   ebccfga,r4
    addis   r4,0,0xfe0b			/* BAS=0xfe0, BS=101 (32MB),BU=11 (R/W), */
    ori     r4,r4,0xc000		/* BW=10 (32 bits) */
    mtdcr   ebccfgd,r4

	/* Done */
    cmpi    cr0,0,r9,0x0            /* check if I cache was off when we */
								    /* started 
    bne     ..ebc_done              /* if it was on, leave on */
    addis   r4,r0,0x0000            /* if it was off, disable */
    mticcr  r4                      /* restore iccr */
    isync

..ebc_done:
    blr


/****************************************************************************
 *
 * Function:     ppcDflush
 * Description:  Flush the data cache. This function does NOT turn off the data
 *               cache after the flush. This function makes use of memory
 *               that hopefully should never be in the data cache under normal
 *               circumstances.
 *                       0xFFFFE000-0xFFFFFFFF
 * Input:        none
 * Output:       none
 */
    .text
    .align  2
    .globl  ppcDflush

ppcDflush:
    addis   r9,r0,0x0002        /* set mask for EE and CE msr bits */
    ori     r9,r9,0x8000
    mfmsr   r12                 /* save msr */
    andc    r9,r12,r9
    mtmsr   r9                  /* disable EE and CE */
    addi    r10,r0,0x0001       /* enable data cache for unused memory */
    mfdccr  r9                  /* region 0xF8000000-0xFFFFFFFF via */
    or      r10,r10,r9          /* bit 31 in dccr */
    mtdccr  r10
    addi    r10,r0,128          /* do loop for # of lines */
    addi    r11,r0,4096         /* D cache set size=4K */
    mtctr   r10
    ori     r10,r0,0xE000       /* start at 0xFFFFE000 */
    add     r11,r10,r11         /* add to get to other side of cache line */
..ppcDflush_loop:
    lwz     r3,0(r10)           /* least recently used side */
    lwz     r3,0(r11)           /* the other side */
    dccci   r0,r11              /* invalidate both sides */
    addi    r10,r10,0x0020      /* bump to next line (32 bytes) */
    addi    r11,r11,0x0020      /* bump to next line (32 bytes) */
    bdnz    ..ppcDflush_loop
    sync                        /* allow memory access to complete */
    mtdccr  r9                  /* restore dccr */
    mtmsr   r12                 /* restore msr */
    blr
