#ifndef _CF_LLD_H
#define _CF_LLD_H

// IDE Register Indices
#define IDE_REG_DATA      0
#define IDE_REG_ERROR     1
#define IDE_REG_FEATURES  1
#define IDE_REG_COUNT     2
#define IDE_REG_REASON    2  // ATAPI
#define IDE_REG_LBALOW    3
#define IDE_REG_LBAMID    4
#define IDE_REG_LBAHI     5
#define IDE_REG_DEVICE    6
#define IDE_REG_STATUS    7
#define IDE_REG_COMMAND   7
#define IDE_REG_EVN_DATA  8
#define IDE_REG_ODD_DATA  9

#define CF_COMMON_MEM_BASE 0x160C0000

#define IDE_STAT_BSY      0x80
#define IDE_STAT_DRDY     0x40
#define IDE_STAT_SERVICE  0x10
#define IDE_STAT_DRQ      0x08
#define IDE_STAT_CORR     0x04
#define IDE_STAT_ERR      0x01

#define IDE_REASON_REL    0x04
#define IDE_REASON_IO     0x02
#define IDE_REASON_COD    0x01

/* flag values */

#define IDE_DEV_PRESENT  1  // Device is present
#define IDE_DEV_PACKET   2  // Supports packet interface
#define IDE_DEV_ADDR48   3  // Supports 48bit addressing

#define CF_REG_RD( _reg_, _value_) \
	_value_ = *((volatile unsigned char *)(CF_COMMON_MEM_BASE + _reg_))

#define CF_REG_RD16( _reg_, _value_) \
	_value_ = *((volatile unsigned short *)(CF_COMMON_MEM_BASE + _reg_))

#define CF_REG_WR( _reg_, _value_) \
	*((volatile unsigned char *)(CF_COMMON_MEM_BASE + _reg_)) = (volatile unsigned char)_value_

#define CF_REG_WR16( _reg_, _value_) \
	*((volatile unsigned short *)(CF_COMMON_MEM_BASE + _reg_)) = (volatile unsigned short)_value_


#define CF_STATUS_RD( _value_) \
	_value_ = *((volatile unsigned char *)(CF_COMMON_MEM_BASE + IDE_REG_STATUS ))

#define CF_CMD_WR( _value_) \
	*((volatile unsigned char *)(CF_COMMON_MEM_BASE + IDE_REG_COMMAND)) = (volatile unsigned char)_value_


/* Alternate status register... */
typedef enum
{
  AltStatusRegBitERR  = 1,
  AltStatusRegBitIDX  = 2,
  AltStatusRegBitCORR = 4,
  AltStatusRegBitDRQ  = 8,
  AltStatusRegBitDSC  = 0x10,
  AltStatusRegBitDF   = 0x20,
  AltStatusRegBitDRDY = 0x40,
  AltStatusRegBitBSY  = 0x80
} IDE_ALTERNATE_STATUS_REGISTER_BITS;

/* Device control register... */
typedef enum
{
  DevControlRegBitNIEN = 2,
  DevControlRegBitSRST = 4,
} IDE_DEVICE_CONTROL_REGISTER_BITS;

/* Device/Head register... */
typedef enum
{
  DevHeadRegBitHeads   = 0xF,
  DevHeadRegBitDEV     = 0x10,
  DevHeadRegBitLBAMode = 0x40,
} IDE_DEVICE_HEAD_REGISTER_BITS;

/* Error register... */
typedef enum
{
  ErrorRegBitAMNF   = 1,
  ErrorRegBitTKNONF = 2,
  ErrorRegBitABRT   = 4,
  ErrorRegBitMCR    = 8,
  ErrorRegBitIDNF   = 0x10,
  ErrorRegBitMC     = 0x20,
  ErrorRegBitUNC    = 0x40,

} IDE_ERROR_REGISTER_BITS;

/* Status register... */
typedef enum
{
  StatusRegBitERR  = 1,
  StatusRegBitIDX  = 2,
  StatusRegBitCORR = 4,
  StatusRegBitDRQ  = 8,
  StatusRegBitDSC  = 0x10,
  StatusRegBitDF   = 0x20,
  StatusRegBitDRDY = 0x40,
  StatusRegBitBSY  = 0x80

} IDE_STATUS_REGISTER_BITS;

/* IDE command definitions */
typedef enum
{
  IdentifyDeviceCmd             = 0xEC,
  ReadSectorWithRetries         = 0x20,
  ReadSectorWithoutRetries      = 0x21,
  WriteSectorWithRetries        = 0x30,
  WriteSectorWithoutRetries     = 0x31,
  InitializeDeviceParameters    = 0x91,
  ExecuteDeviceDiagnostic       = 0x90,
  DMAReadSectorWithRetries      = 0xC8,
  DMAReadSectorWithoutRetries   = 0xC9,
  DMAWriteSectorWithRetries     = 0xCA,
  DMAWriteSectorWithoutRetries  = 0xCB,

} IDE_COMMANDS;


typedef struct ide_identify_data_t_ 
{
    unsigned short general_conf;         // 00    : general configuration
    unsigned short num_cylinders;        // 01    : number of cylinders (default CHS trans)
    unsigned short reserved0;            // 02    : reserved
    unsigned short num_heads;            // 03    : number of heads (default CHS trans)
    unsigned short num_ub_per_track;     // 04    : number of unformatted bytes per track
    unsigned short num_ub_per_sector;    // 05    : number of unformatted bytes per sector
    unsigned short num_sectors;          // 06    : number of sectors per track (default CHS trans)
    unsigned short num_card_sectors[2];  // 07-08 : number of sectors per card
    unsigned short reserved1;            // 09    : reserved
    unsigned short serial[10];           // 10-19 : serial number (string)
    unsigned short buffer_type;          // 20    : buffer type (dual ported)
    unsigned short buffer_size;          // 21    : buffer size in 512 increments
    unsigned short num_ECC_bytes;        // 22    : number of ECC bytes passed on R/W Long cmds
    unsigned short firmware_rev[4];      // 23-26 : firmware revision (string)
    unsigned short model_num[20];        // 27-46 : model number (string)
    unsigned short rw_mult_support;      // 47    : max number of sectors on R/W multiple cmds
    unsigned short reserved2;            // 48    : reserved
    unsigned short capabilities;         // 49    : LBA, DMA, IORDY support indicator
    unsigned short reserved3;            // 50    : reserved
    unsigned short pio_xferc_imting;     // 51    : PIO data transfer cycle timing mode
    unsigned short dma_xferc_timing;     // 52    : single word DMA data transfer cycle timing mode
    unsigned short cur_field_validity;   // 53    : words 54-58 validity (0 == not valid)
    unsigned short cur_cylinders;        // 54    : number of current cylinders
    unsigned short cur_heads;            // 55    : number of current heads
    unsigned short cur_spt;              // 56    : number of current sectors per track
    unsigned short cur_capacity[2];      // 57-58 : current capacity in sectors
    unsigned short mult_sectors;         // 59    : multiple sector setting
    unsigned short lba_total_sectors[2]; // 60-61 : total sectors in LBA mode
    unsigned short sw_dma;               // 62    : single word DMA support
    unsigned short mw_dma;               // 63    : multi word DMA support
    unsigned short apio_modes;           // 64    : advanced PIO transfer mode supported
    unsigned short min_dma_timing;       // 65    : minimum multiword DMA transfer cycle
    unsigned short rec_dma_timing;       // 66    : recommended multiword DMA cycle
    unsigned short min_pio_timing;       // 67    : min PIO transfer time without flow control
    unsigned short min_pio_iordy_timing; // 68    : min PIO transfer time with IORDY flow control
//  unsigned short reserved4[187];       // 69-255: reserved
} ide_identify_data_t;


typedef struct ide_identify_disk_t_
{
	unsigned char	serial[20+1];
	unsigned char	firmware_rev[8+1];
	unsigned char	model_num[40+1];
	unsigned int	cylinders_num;
	unsigned int	heads_num;
	unsigned int	sectors_num;
	unsigned int	lba_sectors_num;

} ide_identify_disk_t;

#endif // CF_LLD_H

