/* bootce.c
 *
 * By David de la Cruz.
 *
 * Modified by Technology Rescue.
 *	12.03.2003 : CSB226 adaptation.
 *	09.05.2003 : changed clear memory calculation.
 *	03.06.2003 : changed ctxAPP to lcd_switch_buffer.
 *	06.06.2003 : added jump_kernel.
 *	11.07.2003 : added HOST IP address and added unzip support.
 *	11.07.2003 : added FileName var.
 *	20.08.2003 : added INCLUDE_FLASHFX.
 *	21.08.2003 : fixed filename.
 *	03.11.2003 : removed copy to ram before unZipping.
 *	06.11.2003 : removed WINCEJUMP variable.
 *	25.11.2003 : added tcpip args read from RTC RAM.
 *	05.01.2004 : removed boot_kernel method and moved kernel init to main.c.
 *	09.01.2004 : cleared reserved memory.
 *
 *	03.03.2005 : CSB625, read ethernet param from environment for WinCE.
 *	21.03.2005 : debugging tcpip params.
 */

#include "config.h"
#include "genlib.h"
#include "ether.h"
#include "cpuio.h"
#include "stddefs.h"
#include "tfs.h"
#include "bootarg.h"
#include "ds1307.h"

unsigned int load_addr;
unsigned int kernel_size;
char FileName[] = CE_FILE_NAME;

extern char *EnvNames[];

#if INCLUDE_UNZIP
unsigned int zip_addr;
#endif

uchar IpAddConv[IP_SIZE];							// To store converted IP addr.
uchar EtherAddConv[ETHERADD_SIZE];					// To store converted Ether Addr.

#if INCLUDE_FLASHFX

/* This determines the size of the stream buffer in each SEFS stream
   control block.
*/
#define SEF_BUFSIZE	0x200

/*
   The file control structure used by the SEFS to track and
   buffer file I/O operations.
*/
typedef struct SEFS_FileIOStructure D_SEF;
typedef D_SEF *D_PSEF;
struct SEFS_FileIOStructure
{
	D_PSEF			psefNext;				/* pointer to next one of these in some list */
	unsigned int	ulFileSize;				/* file size from DIR */
	unsigned int	ulCurrentFilePosition;	/* byte offset into file on disk */
	unsigned int	ulCurrentVBFOffset;		/* where VBF thinks we are in the file */
	unsigned short	rgfStreamInfo;			/* array of stream state bits */
	unsigned short	uStartingCluster;		/* starting cluster of file on disk */
	unsigned short	uCurrentCluster;		/* current cluster number being accessed on disk for this file */
	unsigned short	uSectorsLeftInCluster;	/* sectors remaining in above cluster */
	unsigned short	uBytesLeftInBuffer;		/* bytes remaining in buffer below */
	unsigned char	rgucFileBuffer[SEF_BUFSIZE];	/* disk I/O buffer for this file */
};

/* Following variables are used for FFX. */
D_PSEF psef;
unsigned int uItemsWritten;
unsigned int uItemsToWrite;
unsigned int uItemSize = 0x200;
unsigned int uItemsRead;
unsigned int uItemsToRead;

char *boot_kernelHelp[] = {
	" This command boots OS from FlashFX disk",
	"",
	"",
	"",
	0
};

char *flash_kernelHelp[] = {
	" This command flashes OS to FlashFX disk",
	"",
	" New kernel should be downloaded with TFTP command.",
	" This command copies ${TFTPGET} bytes from ${APPRAMBASE} address.",
	0
};

int boot_kernel(void)
{
	unsigned int memaddr_to_clear;
	unsigned int memsize_to_clear;
	int	error, count;
	TCPIP_PARAMS *pTcpIpParams;
	char *TcpIpParamsDestAddr;
	char *env;

	printf("\nLoading OS from FFX disk...\n");

	if(!ffx_init())
	{	printf("FFX disk can\'t be interfaced.\n");
		while(1);	// loop forever.
	}

	psef = (D_PSEF)sefopen((unsigned char *)(FileName),(unsigned char *)("rb"));
	if(psef)
	{
#if INCLUDE_UNZIP
		zip_addr = strtol(getenv("ZIPRAMBASE"),0,16);	// 16 = base16.
#endif
		load_addr = strtol(getenv("APPRAMBASE"),0,16);	// 16 = base16.
		printf("%s opened.\n", FileName);
		uItemsToRead = (unsigned int)(((psef->ulFileSize-1) / uItemSize)+1);
#if INCLUDE_UNZIP
		uItemsRead = sefread(zip_addr, uItemSize, uItemsToRead, psef);
#else
		uItemsRead = sefread(load_addr, uItemSize, uItemsToRead, psef);
#endif
		printf("Blocks read : %ld\n", uItemsRead);

#if INCLUDE_UNZIP
		/* unzip from ZIPBASERAM to APPRAMBASE. */
		kernel_size = unZip(zip_addr,psef->ulFileSize,load_addr,99999999);
		printf("\nDecompressed %ld bytes from 0x%lx to %d bytes at 0x%lx.\n",
		psef->ulFileSize,(unsigned long)zip_addr,kernel_size,(unsigned long)load_addr);
		memaddr_to_clear = load_addr+kernel_size;
#else
		memaddr_to_clear = load_addr+psef->ulFileSize;
#endif
		memsize_to_clear = (LCD_FDR_ADD_CE)- memaddr_to_clear;
		if(uItemsRead)
		{	psef->ulFileSize=0;	// Don't update file entry.
			if(sefclose(psef) == 0)
				printf("%s closed.\n", FileName);;
		}

		printf("\nJumping to OS ...\n");

		lcd_switch_buffer();				// called switch buffer.

		/* init remaining memory to 0. */
		memset(memaddr_to_clear, 0x00, memsize_to_clear);

		/* read load_addr from APPRAMBASE shell var. */
		load_addr = strtol(getenv("APPRAMBASE"),0,16);	// 16 = base16.

		/* Init tcpip params. */
		/* IPADDR is mandatory. */
		pTcpIpParams = (TCPIP_PARAMS *)(load_addr + TCPIP_PTR_OFFSET);
		IpToBin(getenv(EnvNames[IP_ADDR_IDX]),IpAddConv);
		memcpy((char *)(&pTcpIpParams->IpAdd), (char *)&IpAddConv, IP_ADDR_SIZE);
	
		TcpIpParamsDestAddr = (char *)&pTcpIpParams->NetMask;
		for(count = NETMASK_IDX; count <ETHERADD_IDX; count++)
		{
			env = (char *)getenv(EnvNames[count]);
			if(env)
			{
				IpToBin(env, IpAddConv);
				memcpy(TcpIpParamsDestAddr, (char *)&IpAddConv, IP_SIZE);
			}
			TcpIpParamsDestAddr += IP_SIZE;
		}
	
		/* ETHERADD is mandatory. */
		EtherToBin(getenv(EnvNames[ETHERADD_IDX]),EtherAddConv);
		memcpy((char *)(pTcpIpParams->MACAdd),(char *)&EtherAddConv, ETHERADD_SIZE);

		/* Finally, jump to kernel. */
		((void (*)(void))(load_addr))();	// Jump to kernel in RAM.

	}
	else
	{
		printf("%s doesn't exist.\n", FileName);
	}
	return 0;
}

int flash_kernel(void)
{
	printf("\nFlashing WinCE.NET kernel to FFX disk...\n");

	if(!ffx_init())
	{	printf("FFX disk can\'t be interfaced.\n");
		while(1);	// loop forever.
	}

	psef = (D_PSEF)sefopen((unsigned char *)(FileName),(unsigned char *)("wb"));
	if(psef)
	{
		printf("%s opened.\n", FileName);
		load_addr = strtol(getenv("APPRAMBASE"),0,16);	// 16 = base16.
		kernel_size = atoi(getenv("TFTPGET"));
		uItemsToWrite = (unsigned int)(((kernel_size-1) / uItemSize)+1);
		uItemsWritten = sefwrite(load_addr, uItemSize, uItemsToWrite, psef);
		printf("Blocks written : %ld\n", uItemsWritten);
		if(uItemsWritten)
		{
			psef->ulFileSize = psef->ulCurrentFilePosition;	// update file size before updationg directory entry.
			if(sefclose(psef) == 0)
				printf("%s closed.\n", FileName);
		}
	}
	else
	{
		printf("FFX disk full.\n");
	}
	return 0;
}

#else

char *boot_kernelHelp[] = {
	" This command does nothing.",
	"",
	"",
	"",
	0
};

char *flash_kernelHelp[] = {
	" This command does nothing.",
	"",
	" Use tftp command as following :",
	" tftp -Ftfsfilename $TFTPSRVR get hostfilename",
	0
};

int boot_kernel(void)
{
	return 0;
}

int flash_kernel(void)
{
	return 0;
}

#endif	/* INCLUDE_FLASHFX */

char *jump_kernelHelp[] = {
	" This command jumps to windows CE kernel",
	"",
	" New kernel should be downloaded with TFTP command.",
	" This command clears memory.",
	0
};

void
jump_kernel(void)
{
	unsigned int memaddr_to_clear;		// To contain start address to clear.
	unsigned int memsize_to_clear;		// To contains # bytes to clear.
	int	count;
	TCPIP_PARAMS *pTcpIpParams;
	char *TcpIpParamsDestAddr;
	char *env;

	printf("\nJumping to OS ...\n");

	/* Get kernel start address. */
	load_addr = strtol(getenv("APPRAMBASE"),0,16);	//  16 = base16.

	/* Get kernel size from TFTPGET env. var. */
	kernel_size = atoi(getenv("TFTPGET"));

	/* Init free memory start address and size to clear. */
	memaddr_to_clear = load_addr+kernel_size;		
	memsize_to_clear = (LCD_FDR_ADD_CE)- memaddr_to_clear;

	/* Switch LCD frame buffer. */
	lcd_switch_buffer();

	/* Init remaining memory to 0. */
	memset((char *)memaddr_to_clear, 0x00, memsize_to_clear);
	memset((char *)RESERVED_ADDR, 0x00, (RESERVED_END_ADDR - RESERVED_ADDR));

	/* read load_addr from APPRAMBASE shell var. */
	load_addr = strtol(getenv("APPRAMBASE"),0,16);	// 16 = base16.

	/* Init tcpip params. */
	/* IPADDR is mandatory. */
	pTcpIpParams = (TCPIP_PARAMS *)(load_addr + TCPIP_PTR_OFFSET);
	IpToBin(getenv(EnvNames[IP_ADDR_IDX]),IpAddConv);
	memcpy((char *)(&pTcpIpParams->IpAdd), (char *)&IpAddConv, IP_ADDR_SIZE);

	TcpIpParamsDestAddr = (char *)&pTcpIpParams->NetMask;
	for(count = NETMASK_IDX; count <ETHERADD_IDX; count++)
	{
		env = (char *)getenv(EnvNames[count]);
		if(env)
		{
			IpToBin(env, IpAddConv);
			memcpy(TcpIpParamsDestAddr, (char *)&IpAddConv, IP_SIZE);
		}
		TcpIpParamsDestAddr += IP_SIZE;
	}

	/* ETHERADD is mandatory. */
	EtherToBin(getenv(EnvNames[ETHERADD_IDX]),EtherAddConv);
	memcpy((char *)(pTcpIpParams->MACAdd),(char *)&EtherAddConv, ETHERADD_SIZE);

	/* Finally, jump to kernel. */
	((void (*)(void))(load_addr))();	// Jump to kernel in RAM.

}
