/*
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:	Ed Sutter
 * email:	esutter@lucent.com
 * phone:	908-582-2351
 */

/* LOOPS_PER_SECOND:
 * Approximately the size of a loop that will cause a 1-second delay.
 */
#define LOOPS_PER_SECOND	20000

#define GLOBALINTBASE 99

/* Flash bank configuration: */
#define FLASH_BANK0_WIDTH		2
#define FLASHBANKS				1
#define FLASH_BANK0_BASE_ADDR	0x400000

/* TFS definitions:
 *	TFSSTART:		Base address in FLASH at which TFS starts.
 *	TFSEND:			Size of the flash space (starting at TFSSTART)
 *					dedicated to TFS.
 *	TFSSPARE:		Location of sector that is used as the spare sector
 *					by TFS for defragmentation.
 *	TFSSPARESIZE:	Size of the spare sector used by TFS for defragmentation.
 *	TFSFLASHWIDTH:	Width (in bytes) of the flash used by TFS.
 *	TFSSECTORCOUNT:	Number of eraseable sectors that TFS covers, not including
 *					the TFSSPARE sector.
 */
#define TFSSTART		0x440000
#define TFSEND			0x4effff
#define TFSSPARE		0x4f0000
#define TFSSPARESIZE	0x10000
#define TFSSECTORCOUNT	11		/* Sectors 6-17 inclusive are used by TFS. */
#define TFSFLASHWIDTH	2
#define TFS_EBIN_ELF	1

/* Width macros used by TFS and FLASH operations.
 * Note that only one of WIDTH8, WIDTH16 and WIDTH32 should be set and the
 * WIDTHXX_NOTUESD macros should reflect the opposite state of the WIDTH
 * macros.
 */
#define	WIDTH8		0
#define	WIDTH16		1
#define	WIDTH32		0
#define	WIDTH8_NOTUSED	1
#define	WIDTH16_NOTUSED	0
#define	WIDTH32_NOTUSED	1

/* This macro is used by the flash code to determine how much of the boot
 * flash is protected.  It is usually 0x20000 or 0x30000, but can vary
 * depending on the size of the boot monitor implementation.
 */
#define FLASH_PROTECT_SIZE	0x40000

/* Specify the type of 68K CPU here. Only one should be defined. */
#define CPU_NAME		"68K-EN302"
#define PLATFORM_NAME	"68EN302 Board"

/* Specify the size of the memory block (in monitor space) that is to be
 * allocated to malloc in the monitor.  Note that this size can be dynamically
 * increased through the HEAPBASE and HEAPSTART shell variables.
 */
#define ALLOCSIZE		8*1024
#define MONSTACKSIZE	2048

/* Specify inclusion of subsystems within the monitor here.
 */
#define INCLUDE_MEMCMDS         1
#define INCLUDE_EDIT            0
#define INCLUDE_DISASSEMBLER    0
#define INCLUDE_VERBOSEHELP		0
#define INCLUDE_UNZIP           0
#define INCLUDE_ETHERNET        0
#define INCLUDE_ICMP			0
#define INCLUDE_TFTP            0
#define INCLUDE_DHCPBOOT        0
#define INCLUDE_TFS             0
#define INCLUDE_TFSAPI          0
#define INCLUDE_TFSCLI          0
#define INCLUDE_TFSSYMTBL       0
#define INCLUDE_TFSSCRIPT       0
#define INCLUDE_XMODEM          1
#define INCLUDE_LINEEDIT        0
#define INCLUDE_EE              0
#define INCLUDE_FLASH           0
#define INCLUDE_MALLOC          1
#define INCLUDE_SHELLVARS       1
#define INCLUDE_GDB  	        0
#define INCLUDE_STRACE          0
#define INCLUDE_CAST            0
#define INCLUDE_STRUCT          0
#define INCLUDE_REDIRECT        0
#define INCLUDE_QUICKMEMCPY     0
#define INCLUDE_PROFILER        0
#define INCLUDE_BBC             0
#define INCLUDE_MEMTRACE        0
#define INCLUDE_SPRCMD          0
#define INCLUDE_DCRCMD          0
#define INCLUDE_TRAPCMD         0
#define INCLUDE_SYSCALLCMD      0
#define INCLUDE_STOREMAC		INCLUDE_FLASH
#define INCLUDE_XSVFCMD			0
#define INCLUDE_WDCMD			0
#define INCLUDE_HWTMR			0
#define INCLUDE_SYSLOG			0
#define INCLUDE_PORTCMD			0
#define INCLUDE_USRLVL			0
#define INCLUDE_JFFS2			0
#define INCLUDE_JFFS2ZLIB		0

/* Inclusion of this next file will make sure that all of the above
 * inclusions are legal; and adjust where necessary.
 */
#include "inc_check.h"
