/*
 * Monitor configuration file for FADS860 Evaluation Platform.
 *
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:	Ed Sutter
 * email:	esutter@lucent.com
 * phone:	908-582-2351
 */

#define FORCE_BSS_INIT	1
#define BOOTROMBASE_OVERRIDE 0xfff00000

/* XBUFCNT & RBUFCNT:
 *	Number of transmit and receive buffers allocated to ethernet.
 *	The total of XBUFCNT+RBUFCNT should not exceed MAXEBDS (defined in 
 *	mpc860.h.
 */
#define XBUFCNT	8
#define RBUFCNT	8

/* LOOPS_PER_SECOND:
 * Approximately the size of a loop that will cause a 1-second delay.
 * This can be guestimated or modified with the sleep -c command at the
 * monitor command line.
 */
#define LOOPS_PER_SECOND	20000

/* Flash bank configuration:
 * This target is configured with one flash bank.
 */
#define FLASH_LOOP_TIMEOUT	10000000
#define SINGLE_FLASH_DEVICE		1
#define FLASHBANKS				1
#define FLASH_BANK0_WIDTH		2
#define FLASH_BANK0_BASE_ADDR	0xff000000
#define FLASH_LARGEST_SECTOR	0x20000
#define FLASH_PROTECT_RANGE		"120-127"
#define FLASH_COPY_TO_RAM		1

/* TFS definitions:
 *	TFSSTART:		Base address in FLASH at which TFS starts.
 *	TFSEND:			End address of TFS in FLASH.
 *	TFSSPARE:		Location of sector that is used as the spare sector
 *					by TFS for defragmentation.
 *	TFSSPARESIZE:	Size of the spare sector used by TFS for defragmentation.
 *	TFSSECTORCOUNT:	Number of eraseable sectors that TFS covers, not including
 *					the TFSSPARE sector.
 */
#define TFSSPARESIZE	FLASH_LARGEST_SECTOR
#define TFSSECTORCOUNT	59
#define TFS_DEVTOT		1
#define TFSSTART		(FLASH_BANK0_BASE_ADDR+0x80000)
#define TFSSPARE		(FLASH_BANK0_BASE_ADDR+0x7e0000)
#define TFSEND			(TFSSPARE-1)
#define TFS_EBIN_ELF	1
#define TFS_VERBOSE_STARTUP	1

/* Specify CPU/PLATFORM type and name so that common code can be used
 * for a similar cpu, on different platforms.
 * The 'TYPE' definition is used for ifdefs in the code and the 'NAME'
 * is used for printfs in the code.
 */
#define CPU_NAME		"PowerPC-852T"
#define PLATFORM_NAME	"PPC852 Board"

/* Specify the size of the memory block (in monitor space) that is to be
 * allocated to malloc in the monitor.  Note that this size can be dynamically
 * increased using the heap extension option in the heap command.
 */
#define ALLOCSIZE		(64*1024)
#define MONSTACKSIZE	(8*1024)

/* Specify inclusion of subsystems within the monitor here.
 * Refer to comments in common/monitor/inc_check.h for details on
 * each of these macros.
 */
#define INCLUDE_MEMCMDS			1
#define INCLUDE_EDIT			1
#define INCLUDE_DISASSEMBLER	1
#define INCLUDE_UNZIP			1
#define USE_FEC					1
#define INCLUDE_ETHERNET		1
#define INCLUDE_ICMP			1
#define INCLUDE_TFTP			1
#define INCLUDE_DHCPBOOT		1
#define INCLUDE_TFS				1
#define INCLUDE_TFSCLI			1
#define INCLUDE_TFSAPI			1
#define INCLUDE_TFSSCRIPT		1
#define INCLUDE_TFSSYMTBL		1
#define INCLUDE_XMODEM			1
#define INCLUDE_LINEEDIT		1
#define INCLUDE_EE				1
#define INCLUDE_FLASH			1
#define INCLUDE_MALLOC			1
#define INCLUDE_SHELLVARS		1
#define INCLUDE_STRACE			1
#define INCLUDE_CAST			1
#define INCLUDE_STRUCT			1
#define INCLUDE_REDIRECT		1
#define INCLUDE_QUICKMEMCPY		1
#define INCLUDE_PROFILER		1
#define INCLUDE_BBC				0
#define INCLUDE_MEMTRACE		1
#define INCLUDE_STOREMAC		0
#define INCLUDE_VERBOSEHELP		1
#define INCLUDE_HWTMR			0
#define INCLUDE_SYSLOG			1
#define INCLUDE_PORTCMD			0
#define INCLUDE_USRLVL			1

/* Inclusion of this next file will make sure that all of the above
 * inclusions are legal; and warn/adjust where necessary.
 */
#include "inc_check.h"
