/* boot.S:
 * The boot.S does almost nothing. There's two reasons for that:
 * 1. This monitor assumes it is booted from some other BRAM-based
 *    boot code internal to the V4.  If it isn't, then this needs to
 *    be remapped to have the reset vector at the top of memory (0xfffffffc).
 * 2. The IP glue of the V4 starts up already configured, so in general,
 *    there's nothing to do except establish the stack.
 * If there is a need to make this  more like a "normal" uMon, then
 * just reuse some of the initialization stuff (i.e. exception handlers)
 * from the WALNUT port.
 *
 * Also note that this file is called "boot.S" instead of the normal
 * "reset.S".  That's because the toolset that comes with Xilinx expects
 * to find boot.o; so we give it one.
 *
 */
    .file   "boot.S"
	.text

	.extern start
	.extern MonStack
	.global reset
	.global warmstart
	.global coldstart
	.global exit

#include "config.h"
#include "warmstart.h"

#define sp	1
#define r0	0
#define r1	1
#define r3	3
#define r7	7
#define r15	15


	.ascii "uMon"

	.balign 0x10 

#include "moncomptr.S"

	.balign 0x10 

#include "etheraddr.S"

	.balign 0x100 

/*********************************************************************
 *
 * coldstart:
 * The reset point.
 */
reset:
exit:
coldstart:
	/* - Invalidate and disable caches
 	 * - Load INITIALIZE (defined in warmstart.h) into whereever it
	 *   needs to be put so that warmstart sees it as an argument
	 *   passed to it.
	 * 
	 * - Jump to warmstart
	 */
	xor		r0,r0,r0	/* clear r0 */
	addi	r3,r0,INITIALIZE
	
/*********************************************************************
 *
 * warmstart:
 * A point callable by C, as warmstart(int type), where
 * 'type' is one of the values defined in warmstart.h.
 */
warmstart:
	/* Minimal CPU/IO Initialization (i.e. chip selects) here.
	 * Stay in assembler here if possible.
	 */
    
/*********************************************************************
 *
 * SPInit:
 * Establish a stack frame.
 */
SPInit:
	/* First initialize stack to point to MonStack+MONSTACKSIZE+64
	 * (an address that is outside the MonStack array), and then
	 * call stkinit().  This loads the MonStack[] array with a known
	 * pattern that allows uMon to later analyze the running stack usage.
	 */
    lis     sp, (MonStack+(MONSTACKSIZE+64))@h
    ori     sp, sp, (MonStack+(MONSTACKSIZE+64))@l
    bl      stkinit
    
	/* Next, re-initialize stack to point to MonStack+MONSTACKSIZE...
	 * This is important because other portions of the code
	 * assume this is where the stack resides.
	 */
    lis     sp, (MonStack+(MONSTACKSIZE-4))@h
    ori     sp, sp, (MonStack+(MONSTACKSIZE-4))@l
    addi    r7, r0, -8
    and     sp, sp, r7      /* 8-byte aligned (EABI spec) */

/*********************************************************************
 *
 * gotoC:
 * This code jumps to the start() function in the monitor
 * and should never return.
 */
gotoC:
	/* - Retrieve the variable passed to warmstart and place it
	 *   whereever it needs to be so that the start() function in
	 *   'C' sees it as an argument.  Note that if FORCE_BSS_INIT
	 *   is defined, then start() will ignore this argument.
	 * - Branch to start().
	 */
    mr      r3,r15          /* Copy r15 to r3. */
    bl      start
    nop
	bl		warmstart		/* umon_start won't return, but just in case */


	.global mySPInit
mySPInit:
    lis     sp, (MonStack+(MONSTACKSIZE-4))@h
    ori     sp, sp, (MonStack+(MONSTACKSIZE-4))@l
    addi    r7, r0, -8
    and     sp, sp, r7      /* 8-byte aligned (EABI spec) */
	blr


	/* PPC errata 213: only for Virtex-4 */
#define CCR0 0x3b3

	.global virtex4_errata_fix
virtex4_errata_fix:
    mfspr   r0,CCR0
    oris    0,0,0x50000000@h
    mtspr   CCR0,r0
	blr

	.global flush_instruction_cache
flush_instruction_cache:
    lis r3, 0
    iccci   0,r3
	isync
	blr
